package com.zoniot.ccrc.commom.utils;

import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.security.mobile.MobileLoginAuthenticationToken;
import org.springframework.security.authentication.AnonymousAuthenticationToken;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;

import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

public class UserUtil {

    /**
     *获取当前用户
     */
    public static LoginUser getCurrentUser() {
        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        if (authentication != null) {
            if (authentication instanceof AnonymousAuthenticationToken) {
                return null;
            }

            if (authentication instanceof UsernamePasswordAuthenticationToken) {
                return (LoginUser) authentication.getPrincipal();
            }

            if(authentication instanceof MobileLoginAuthenticationToken){
                return (LoginUser) authentication.getPrincipal();
            }
        }

        return null;
    }



    /**
     *获取用户当前站点id
     */
    public static List<Integer> getUserCurrentSiteIds() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        if (loginUser != null && loginUser.getSiteId() != null) {
            List<Integer> siteList = newArrayList();
            siteList.add(loginUser.getSiteId());
            return siteList;
        }else {
            return null;
        }
    }

     /**
        * @Description: 获取当前登录用户的CustommerId
        * @Param:
        * @return:
        * @Author: Yangyang.Wang
        * @Date: 2020/4/16
      */
    public static List<Integer> getUserCurrentCustommerId() {
        LoginUser loginUser = UserUtil.getCurrentUser();
        if (loginUser != null && loginUser.getCustomerId() != null) {
            List<Integer> customerList = newArrayList();
            customerList.add(loginUser.getCustomerId());
            return customerList;
        }else {
            return null;
        }
    }

    public static String getUsername(){
        LoginUser loginUser = UserUtil.getCurrentUser();
        if (loginUser != null) {
            return loginUser.getUsername();
        }else {
            return "system";
        }
    }


}
