package com.zoniot.ccrc.service.impl;

import com.alibaba.fastjson.JSON;
import com.github.pagehelper.PageHelper;
import com.zoniot.ccrc.commom.exception.ServiceException;
import com.zoniot.ccrc.commom.model.CommunityData;
import com.zoniot.ccrc.commom.model.Pagination;
import com.zoniot.ccrc.commom.utils.TreeUtil;
import com.zoniot.ccrc.commom.utils.UserUtil;
import com.zoniot.ccrc.commom.utils.Util;
import com.zoniot.ccrc.dao.AreaMapper;
import com.zoniot.ccrc.dao.BuildingMapper;
import com.zoniot.ccrc.dao.CommunityMapper;
import com.zoniot.ccrc.dao.OrganizationMapper;
import com.zoniot.ccrc.dto.BuildingSelectDto;
import com.zoniot.ccrc.dto.CommunityDto;
import com.zoniot.ccrc.dto.LoginUser;
import com.zoniot.ccrc.entity.Area;
import com.zoniot.ccrc.entity.Community;
import com.zoniot.ccrc.service.CommunityService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

@Slf4j
@Service
public class CommunityServiceImpl implements CommunityService {

    @Resource
    private CommunityMapper communityMapper;
    @Resource
    private BuildingMapper buildingMapper;

    @Resource
    private AreaMapper areaMapper;
    @Resource
    private OrganizationMapper organizationMapper;

    @Override
	public List<BuildingSelectDto> regionSearch(Integer siteId , String name) {
        List<Integer> ids = newArrayList();
        List<BuildingSelectDto> buildingSelectDtoArrayList = newArrayList();
        List<CommunityDto> communityList = communityMapper.findBySiteId(siteId, null);

        if (communityList != null && communityList.size() > 0) {
            communityList.forEach(community -> {
                ids.add(community.getProvince());
                ids.add(community.getCity());
                ids.add(community.getRegion());
            });
        }
        //查询所有建筑的区域
        List<Area> areaList = newArrayList();
        if (ids.size() > 0) {
            areaList = areaMapper.findByIdsAndName(Util.removeDuplicate(ids),name);
        }
        if (areaList != null && areaList.size() > 0) {
            areaList.forEach(area -> {
                BuildingSelectDto buildingSelectDto = new BuildingSelectDto();
                buildingSelectDto.setCode(area.getId());
                buildingSelectDto.setPid(area.getParentId());
                buildingSelectDto.setName(area.getName());
                buildingSelectDto.setStatus(0);
                if(area.getLevelType() == 1)buildingSelectDto.setDimensionCode("PROVINCE");
                if(area.getLevelType() == 2)buildingSelectDto.setDimensionCode("CITY");
                if(area.getLevelType() == 3)buildingSelectDto.setDimensionCode("REGION");

                buildingSelectDtoArrayList.add(buildingSelectDto);
            });
        }
		return buildingSelectDtoArrayList;
	}


    @Override
    public int insert(Community community){
        community.setCreateBy("system");
        community.setDateCreate(new Date());
        community.setUpdateBy("system");
        community.setDateUpdate(new Date());
        return communityMapper.insert(community);
    }

    @Override
    public int insertSelective(Community community){
        community.setCreateBy(UserUtil.getCurrentUser().getUsername());
        community.setDateCreate(new Date());
        community.setUpdateBy(UserUtil.getCurrentUser().getUsername());
        community.setDateUpdate(new Date());
        return communityMapper.insertSelective(community);
    }

    @Override
    public int insertList(List<Community> communitys){
        return communityMapper.insertList(communitys);
    }

    @Override
    public int updateByPrimaryKeySelective(Community community){
        community.setUpdateBy("system");
        community.setDateUpdate(new Date());
        return communityMapper.updateByPrimaryKeySelective(community);
    }

	@Override
	public List<CommunityDto> queryList(Community community) {
		return communityMapper.queryList(community);
	}

    @Override
    public List<CommunityDto> queryCommunity(Community community) {
        return communityMapper.queryCommunity(community,null);
    }

    @Override
    public Pagination<CommunityDto> getCommunityPage(String communityName, Integer province, Integer city, Integer region,Integer orgId, int pageNum, int pageSize) {
        LoginUser loginUser = UserUtil.getCurrentUser();

        PageHelper.startPage(pageNum, pageSize);
        List<CommunityDto> list = communityMapper.getList(loginUser.getSiteId(), null, communityName, province, city, region, orgId);
        return new Pagination<>(list);

    }

    @Override
	public List<CommunityDto> queryCommunityTree(Integer siteId, String communityName) {
    	List<CommunityDto> rtnList = communityMapper.findCommunityContainBuilding(siteId, communityName);
		return rtnList;
	}
    @Override
    public void addCommunity(CommunityDto communityDto) {
        log.info("Begin AddCommunity Community " + communityDto);
        LoginUser loginUser = UserUtil.getCurrentUser();
        Integer siteId = loginUser.getSiteId();

        if(communityDto.getProvince() == null || communityDto.getProvince() == 0){
            throw new ServiceException(-900, "获取省编码错误，请重新输入");
        }
        if(communityDto.getCity() == null || communityDto.getCity() == 0){
            throw new ServiceException(-900, "获取市编码错误，请重新输入");
        }
        if(communityDto.getRegion() == null || communityDto.getRegion() == 0){
            throw new ServiceException(-900, "获取区编码错误，请重新输入");
        }
        int resultName = communityMapper.findByNameUnique(siteId,null,communityDto.getProvince(),communityDto.getCity(),communityDto.getRegion(), communityDto.getName());
        if(resultName > 0) {
            throw new ServiceException(-900, "名称已经存在");
        }
        communityDto.setSiteId(siteId);
        communityDto.setCustomerId(loginUser.getCustomerId());

        communityDto.setStatus(1);
        //获取小区最大值，然后加1
        String code = "001";
        /*List<Community> communities = communityMapper.getMaxCodeBySiteId(communityDto.getSiteId());
        if(communities.size()==1)
        {
            if (communities.get(0) != null) {
                code =String.format("%03d",Integer.valueOf(communities.get(0).getCode()));
            }else {
                code = String.format("%03d",Integer.valueOf("1"));
            }

        }
        communityDto.setCode(code);*/
        this.insert(communityDto);
        log.info("End AddCommunity");
    }

    @Override
    public void editCommunity(Community community) {
        log.info("Begin EditCommunity Community " + community);
        LoginUser loginUser = UserUtil.getCurrentUser();
        Integer siteId = loginUser.getSiteId();

        if(community.getProvince() == null || community.getProvince() == 0){
            throw new ServiceException(-900, "获取省编码错误，请重新输入");
        }
        if(community.getCity() == null || community.getCity() == 0){
            throw new ServiceException(-900, "获取市编码错误，请重新输入");
        }
        if(community.getRegion() == null || community.getRegion() == 0){
            throw new ServiceException(-900, "获取区编码错误，请重新输入");
        }
        int resultName = communityMapper.findByNameUnique(siteId, community.getId(), community.getProvince(), community.getCity(), community.getRegion(), community.getName());
        if(resultName > 0) {
            throw new ServiceException(-900, "名称已经存在");
        }
        this.updateByPrimaryKeySelective(community);
        log.info("End EditCommunity");
    }

    @Override
    public void deleteCommunity(Integer communityId) {
        log.info("Begin DeleteCommunity CommunityId" + communityId);
        int reslut = buildingMapper.countBycCommunityId(communityId);
        if(reslut > 0){
            throw new ServiceException(-900,"该小区已经存在建筑，请先将建筑删除");
        }

        Community community = new Community();
        community.setId(communityId);
        community.setStatus(0);
        this.updateByPrimaryKeySelective(community);
        log.info("End DeleteCommunity");
    }

    @Override
    public List<BuildingSelectDto> getAreaTree() {
        LoginUser loginUser = UserUtil.getCurrentUser();

        List<Integer> ids = newArrayList();
        List<BuildingSelectDto> buildingSelectDtoArrayList = newArrayList();

        List<CommunityDto> communityList =newArrayList();
        communityList = communityMapper.findBySiteId(loginUser.getSiteId(),null);


        if (communityList != null && communityList.size() > 0) {
            communityList.forEach(community -> {
                ids.add(community.getProvince());
                ids.add(community.getCity());
                ids.add(community.getRegion());
            });
        }
        //查询所有建筑的区域
        List<Area> areaList = newArrayList();
        if (ids.size() > 0) {
            areaList = areaMapper.findByIds(Util.removeDuplicate(ids));
        }
        if (areaList != null && areaList.size() > 0) {
            areaList.forEach(area -> {
                BuildingSelectDto buildingSelectDto = new BuildingSelectDto();
                buildingSelectDto.setCode(area.getId());
                buildingSelectDto.setPid(area.getParentId());
                buildingSelectDto.setName(area.getName());
                buildingSelectDto.setStatus(0);
                if(area.getLevelType() == 1)buildingSelectDto.setDimensionCode("PROVINCE");
                if(area.getLevelType() == 2)buildingSelectDto.setDimensionCode("CITY");
                if(area.getLevelType() == 3)buildingSelectDto.setDimensionCode("REGION");

                buildingSelectDtoArrayList.add(buildingSelectDto);
            });
        }
        return TreeUtil.getBuildingSelect(buildingSelectDtoArrayList, 100000, "", 1);
    }

    @Override
    public Community findByName(Integer siteId, String communityName) {
        return communityMapper.findByName(siteId,communityName);
    }

    @Override
    public Community findById(Integer communityId) {
        return communityMapper.findById(communityId);
    }

    @Override
    public List<Community> findBySiteId(Integer siteId) {
        return communityMapper.getBySiteId(siteId);
    }

    @Override
    public List<Community> getMaxCodeBySiteId(Integer siteId){
        return  communityMapper.getMaxCodeBySiteId(siteId);
    }

    @Override
    public Community findByNameV2(Integer siteId, Integer province, Integer city, Integer region, String communityName) {
        return communityMapper.findByNameV2(siteId,province,city,region,communityName);
    }

    @Override
    public Community findCacheByName(Integer siteId, String communityName) {
        return communityMapper.findByName(siteId,communityName);
    }

    @Override
    public List<Community> getOrgCommunity(Integer orgId, Integer areaId) {

        return communityMapper.getOrgCommunity(orgId,areaId,UserUtil.getCurrentUser().getSiteId());
    }

    @Override
    public void syncCommunity(CommunityData communityData) {
        Integer siteId =  organizationMapper.findSiteId(communityData.getCustomerId());
        Community communityTemp = communityMapper.findById(communityData.getId());

        Community community = new Community();
        community.setId(communityData.getId());
        community.setSiteId(siteId);
        community.setName(communityData.getName());
        community.setCode(communityData.getCode());
        community.setCustomerId(communityData.getCustomerId());
        community.setProvince(communityData.getProvince());
        community.setCity(communityData.getCity());
        community.setRegion(communityData.getRegion());
        community.setLongitude(communityData.getLongitude());
        community.setLatitude(communityData.getLatitude());
        community.setDistrictId(communityData.getDistrictId());
        community.setAddress(communityData.getAddress());
        community.setRemark(communityData.getRemark());
        community.setStatus(communityData.getStatus());
        community.setDateCreate(communityData.getDateCreate());
        community.setDateUpdate(communityData.getDateUpdate());
        community.setCreateBy("system");
        community.setUpdateBy("system");
        community.setOrgId(communityTemp!=null?communityTemp.getOrgId():null);
        log.info("syncCommunity community {}", JSON.toJSONString(community));
        communityMapper.replaceSelective(community);

    }
}
